'''
common method
'''

import os
import json
import socket
from loguru import logger
from script_client.common.my_requests import MyRequests
from script_client.const import config
import shutil

def replace_str(s):
    s = s.replace('\r\n','')
    s = s.replace('\n', '')
    return s

def get_local_ip():
    local_ip = ""
    try:
        socket_objs = [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]
        ip_from_ip_port = [(s.connect(("8.8.8.8", 53)), s.getsockname()[0], s.close()) for s in socket_objs][0][1]
        ip_from_host_name = [ip for ip in socket.gethostbyname_ex(socket.gethostname())[2] if not ip.startswith("127.")][:1]
        local_ip = [l for l in (ip_from_ip_port, ip_from_host_name) if l][0]
    except (Exception) as e:
        print("get_local_ip found exception : %s" % e)
    return local_ip if("" != local_ip and None != local_ip) else socket.gethostbyname(socket.gethostname())



def is_none(kwargs):
    '''
    0、空、None、False 外都算 True
    :param kwargs:
    :return:
    '''
    ll = []
    for k,v in kwargs.items():
        ll.append(v)
    if all(ll):
        return False
    return True



def del_path(path):
    '''
    删文件夹，以及文件夹内的文件
    :param path:
    :return:
    '''
    try:
        shutil.rmtree(path)
    except Exception as e:
        logger.error(f"清空 {path} 出错 ：{e}")


def del_all_file(path):
    '''
    删目录下所有文件，包括子目录内
    :param path:
    :return:
    '''
    for i in os.listdir(path):
        path_file = os.sep.join([path, i])
        if os.path.isfile(path_file):
            os.remove(path_file)
        else:
            del_all_file(path_file)


def robot_send_message(msg_type='text',message=''):
    try:
        if config.robot_url != '':
            mr = MyRequests(timeout=config.http_request_timeout)
            url = config.robot_url
            json = {
                "msg_type": msg_type,
                "content": {
                    "text": message
                }
            }
            rep = mr.run(method='post', url=url, json=json)
            if isinstance(rep, str) or rep.status_code != 200 or rep.json()['StatusMessage'] != 'success':
                return False
            return True
    except Exception as e:
        logger.error(f"{e}")
        return False



def regist_app(app_code,moniter_path):
    url = config.regist_server_url
    data = {
        'app_code':app_code,
        'moniter_path':moniter_path,
        'ip':config.local_ip,
        'port':config.local_port,
        'tag':{},
        'name':config.client_name
    }
    try:
        mr = MyRequests(config.http_request_timeout)
        rep = mr.run(url=url,method='post',json=data)
        if isinstance(rep, str) or rep.status_code != 200:
            msg = f"注册 {app_code} 失败 - 接口返回 = {rep}"
            logger.error(msg)
            return msg
        else:
            # logger.info(f"注册成功--{data}")
            return True
    except Exception as e:
        logger.error(f'注册{app_code}失败:{str(e)}')
        return f'注册{app_code}失败:{str(e)}'




def str_to_dict(str):
    str = str.replace(r'\r\n','')
    str = str.replace("'", '"')
    return json.loads(str)




if "__main__" == __name__ :
    del_path(r'D:\1\2 python code\script_client\test_script\1')




import os
import sys
import json
import psutil
import platform
import tornado.web
import tornado.ioloop
import tornado.httpserver
import tornado.gen as gen
from loguru import logger
from script_client import func_map
from script_client import file_manage
from script_client import run_script
from script_client.const import config
from script_client.common.common_method import is_none,regist_app



class UploadHandler(tornado.web.RequestHandler):
    def get(self):
        self.render('upload.html')

    def post(self):
        post_date = {}
        try:
            post_date['app_code'] = self.get_body_argument('app_code')
            post_date['moniter_path'] = self.get_body_argument('moniter_path')
            post_date['script_file'] = self.request.files.get('file_obj', None)
            if is_none(post_date):
                self.write({'code': 0, 'msg': '都是必填的'})
                return
            if post_date['script_file'][0].filename.split('.')[-1] != 'py':
                self.write({'code': 0, 'msg': '只能上传.py文件'})
                return
            path = file_manage.find_file(app_code=post_date['app_code'], moniter_path=post_date['moniter_path'])
            if path == False:#不存在才注册
                regist_app_result = regist_app(post_date['app_code'], post_date['moniter_path'])
                if isinstance(regist_app_result, str):
                    self.write({'code': 0, 'msg': regist_app_result})
                    return
            save_file_result = file_manage.save_file(post_date)
            if isinstance(save_file_result, str):
                self.write({'code': 0, 'msg': save_file_result})
                return
            self.write({'code': 1, 'msg': '上传成功'})
        except Exception as e:
            self.write({'code': 0, 'msg': '上传失败：' + str(e)})


class ListenerHandler(tornado.web.RequestHandler):
    def set_default_headers(self):
        self.set_header('Content-Type', 'application/json')

    @gen.coroutine
    def post(self):
        try:
            data = json.loads(self.request.body)
            if not all((data['app_code'],data['moniter_path'],data['http_body'])):
                logger.error(f"有参数为空 data={data}")
                self.write({'code': 0, 'msg': f"有参数为空"})
                return
            if run_script.q.full():
                logger.warning(f"{config.local_ip}:{config.local_port}的队列已满")
                self.write({'code': 0, 'msg': f"{config.local_ip}:{config.local_port}的队列已满"})
                return
            fun_map = func_map.fm.get_map()
            if data['app_code'] in fun_map:
                if data['moniter_path'] in fun_map[data['app_code']]:
                    data['func_info'] = fun_map[data['app_code']][data['moniter_path']]
                    run_script.q.put(data)
                    logger.debug(f"q队列长度 = {run_script.q.qsize()}")
                    self.write({'code': 1, 'msg': 'success'})
                    return
            logger.info(f"没命中函数，现在注册的函数有{fun_map}")
            self.write({'code': 0, 'msg': '没命中函数'})
        except Exception as e:
            self.write({'code': 0, 'msg': f'{e}'})


def run_web_server():
    logger.info(f"启动webserver...")
    handlers =[
        (r"/", UploadHandler),
        (r"/receive_data", ListenerHandler),
    ]
    setting = dict(
        template_path=os.path.join(os.path.dirname(__file__), "templates"),
        static_path=os.path.join(os.path.dirname(__file__), "static"),
    )
    app = tornado.web.Application(handlers,**setting)
    httpServer = tornado.httpserver.HTTPServer(app)
    httpServer.bind(int(config.local_port))
    if (platform.system() != "Windows"):
        httpServer.start(1)
    else:#目前情况一个实例绝对够了
        httpServer.start(1)
    tornado.ioloop.IOLoop.current().start()

def _kill_terminate(p):
    # p.terminate()
    p.kill()


def stop_web_server():
    if os.path.exists('pid.log') == False:
        logger.info('没有pid.log')
        sys.exit(0)
    with open("pid.log", 'r') as fp:
        pid = int(fp.read())
    pidc = psutil.Process(pid)
    # ppid = pid.parent()
    # _kill_terminate(ppid)
    _kill_terminate(pidc)



if __name__ == "__main__":
    run_web_server()



import queue
import json
import requests
import traceback
import threading
import subprocess
from loguru import logger
from threading import Timer
from script_client.const import config
from script_client.common.my_requests import MyRequests


# 用来表示终止的特殊对象
sentinel = object()
# 先进先出
q = None


def _send_resurl(data):
    url = config.result_url
    logger.debug(f"{json.dumps(data)}")
    mr = MyRequests(config.http_request_timeout)
    headers={'Content-Type':'application/json'}
    try:
        rep = mr.run(url=url, method='post', headers=headers,json=json.dumps(data))
        if isinstance(rep, requests.models.Response):
            if rep.status_code != 200:
                logger.error(f"回传脚本验证结果失败 - status_code = {rep.status_code}")
        else:
            logger.error(f"回传脚本验证结果失败 -{rep}")
    except Exception as e:
        logger.error(f'回传脚本验证结果失败:{e}')
    pass


def _timeout_callback(p):
    try:
        p.kill()
        logger.warning(f"timeout, killed --> {p.pid}")
    except Exception as error:
        logger.error(error)


def _run(data, shell=False, stdin=subprocess.PIPE,
         stdout=subprocess.PIPE, stderr=subprocess.PIPE, timeout=config.py_timeout):
    _process = subprocess.Popen(
        data['cmd'], shell=shell,
        stdin=stdin, stdout=stdout, stderr=stderr
    )
    _timer = Timer(timeout, _timeout_callback, [_process])
    _timer.start()
    try:
        stdout, stderr = _process.communicate()
        # logger.warning(type(stdout))
        # logger.warning(stderr)
        exit_code = _process.returncode
        data = {
            'exit_code': exit_code,
            'stdout': str(stdout, encoding="utf-8"),
            'stderr': str(stderr, encoding="utf-8"),
            'app_code': data['app_code'],
            'moniter_path': data['moniter_path'],
        }
        # exit_code ：0=ok,1=err
        _send_resurl(data)
    except Exception as e:
        logger.error(f"process过程出错：{e}")
    finally:
        _timer.cancel()


class MyThread(threading.Thread):
    def __init__(self, queue):
        threading.Thread.__init__(self)
        self.queue = queue
        self.start()

    def run(self):
        while True:
            try:
                # block=False 没任务弹异常，=True且有timeout等timeout，过了弹异常，没timeout死等
                data = self.queue.get()
                if data is sentinel:
                    self.queue.put(sentinel)
                    break
                # _run(data)
                ret_data = {
                    'app_code': data['app_code'],
                    'moniter_path': data['moniter_path'],
                    'result':1,  # 0失败，1通过
                    'result_body': {
                        'type': 0,  # 1=过程异常，2=断言异常,0=无异常
                        'stdout':data['http_body'],
                        'stderr':'',
                    }
                }
                try:
                    data['func_info'][0](data)
                except AssertionError as e:
                    ret_data['result']=0
                    ret_data['result_body']['type'] = 2
                    ret_data['result_body']['stderr'] = repr(e)
                except Exception as e:
                    ret_data['result'] = 0
                    ret_data['result_body']['type'] = 1
                    ret_data['result_body']['stderr'] = repr(e)
                finally:
                    _send_resurl(ret_data)
                    pass
                self.queue.task_done()
            except Exception as e:
                logger.info(f"{e}")


class MyThreadPool():
    def __init__(self, queue, size):
        self.queue = queue
        self.pool = []
        for i in range(size):
            self.pool.append(MyThread(queue))


def init_thread_pool():
    global q
    q = queue.Queue(config.max_task)
    MyThreadPool(queue=q, size=config.max_thread)
    logger.info(f"线程池初始化完成")


if __name__ == '__main__':
    pass




"""
my requests
"""

import requests
from loguru import logger

class MyRequests:
    """
    封装requests
    """
    def __init__(self, timeout=6):
        """
        init
        :param timeout: 超时时间
        """
        self.timeout = timeout

    def get(self, url, **DataAll):
        """
        get请求
        :param url: url
        :param DataAll:all data
        :return:text
        """
        params = DataAll.get('params')
        headers = DataAll.get('headers')
        try:
            resp = requests.get(url=url, params=params, headers=headers, timeout=self.timeout)
            return resp
        except Exception as ex:
            # logger.error(f"url={url}")
            # logger.error(f"headers={headers}")
            # logger.error(f"params={params}")
            # logger.error(str(ex))
            return f"err = {ex}"

    def post(self, url, **DataAll):
        '''
        post请求
        :param url: url
        :param DataAll:all data
        :return:text
        '''
        params = DataAll.get('params')
        headers = DataAll.get('headers')
        data = DataAll.get('data')
        json = DataAll.get('json')
        files = DataAll.get('files')
        try:
            resp = requests.post(url=url, params=params, headers=headers, data=data,
                                 json=json, files=files, timeout=self.timeout)
            return resp
        except Exception as ex:
            # logger.error(f"url={url}")
            # logger.error(f"headers={headers}")
            # logger.error(f"params={params}")
            # logger.error(f"data={data}")
            # logger.error(f"json={json}")
            # logger.error(f"files={files}")
            # logger.error(str(ex))
            return f"err = {ex}"

    def run(self, url, method, **DataAll):
        """
        run
        :param url: url
        :param method: post or get
        :param DataAll: all data
        :return: resp
        """
        resp = None
        method = method.lower()
        if method == 'get':
            resp = self.get(url, **DataAll)
        elif method == 'post':
            resp = self.post(url, **DataAll)
        else:
            logger.error("method的值不正确")
        return resp


##############################################################################
if __name__ == '__main__':
    url = ''
    headers = {
        'Content-Type': 'application/json;charset=utf-8',
    }
    json={}
    mr = MyRequests()
    t = mr.run(url=url, method='post', json=json, headers=headers)
    print(t.status_code)
    print(t.json())





import os
from script_client.const import config
from loguru import logger



def find_file(**k):
    app_code = k['app_code']
    moniter_path = k['moniter_path']
    if '/' in k['moniter_path']:
        moniter_path = moniter_path.replace('/','-')
        file_name = moniter_path+'.'+'py'
    else:
        moniter_path = k['moniter_path'].split('#')[0]
        file_name = k['moniter_path'].split('#')[1]+'.'+'py'
    path = os.sep.join([config.test_script_dir,app_code,moniter_path,file_name])
    if os.path.exists(path):
        return path
    else:
        return False



def save_file(post_date):
    try:
        root_folder = config.test_script_dir #root
        app_code_folder = post_date['app_code']# app_code
        moniter_path_folder = post_date['moniter_path']# moniter_path
        file_name = post_date['script_file'][0].filename # name.py
        if '/' in moniter_path_folder:# url
            moniter_path_folder = moniter_path_folder.replace('/','-')
            file_name = moniter_path_folder+'.'+post_date['script_file'][0].filename.split('.')[-1]
        path = os.sep.join([root_folder,app_code_folder,moniter_path_folder])
        if not os.path.exists(path):
            os.makedirs(path)
        # tornado save file
        for meta in post_date['script_file']:
            with open(os.sep.join([path,file_name]), 'wb+') as f:
                f.write(meta['body'])
        return True
    except Exception as e:
        msg = f"保存文件过程出错：{str(e)}"
        logger.error(msg)
        return msg










class Config:
    def __init__(self):
        self.local_ip='127.0.0.1'
        self.local_port = 7788
        self.client_name = 'xyz'
        self.regist_server_url = r'https://www.baidu.com/'
        self.heart_beat_url = r'https://www.baidu.com/'
        self.result_url = r'https://www.baidu.com/'
        self.http_request_timeout = 1
        self.max_thread = 100
        self.max_task = 2000
        self.test_script_dir = r'D:\1\ts'
        self.py_timeout = 15
        self.robot_url = ''
        self.work_path=''
        pass







config = Config()




from loguru import logger
from script_client.func_map import fm



def register(app='',path='',timeout=0):
    '''
    注册为回调函数
    :param app: app_code
    :param path: path
    :param timeout: timeout 本期没做无效
    :return:
    '''
    def deco(func):
        fm.add_map(app=app,path=path,func=func,timeout=timeout)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)
        return wrapper
    return deco



import os
from loguru import logger
from script_client.func_map import fm
import requests
import importlib
from script_client import func_map
from script_client.common.my_requests import MyRequests
from script_client.common.common_method import robot_send_message
from apscheduler.schedulers.background import BackgroundScheduler
from script_client.const import config


scheduler = BackgroundScheduler()


@scheduler.scheduled_job('interval',id='heart_beat', seconds=30)
def send_heart_beat():
    try:
        mr = MyRequests(timeout=config.http_request_timeout)
        url = config.heart_beat_url
        for k in func_map.fm.get_map():
            data = {"app_code": str(k)}
            rep = mr.run(url=url, method='post', json=data)
            if isinstance(rep, requests.models.Response):
                if rep.status_code != 200:
                    msg = f"心跳发送失败 - app = {str(k)} status_code = {rep.status_code}"
                    logger.error(msg)
                    robot_send_message(message=msg)
            else:
                logger.error(f"心跳发送失败 app = {str(k)} ， {rep}")
                robot_send_message(message='心跳发送失败')


    except Exception as e:
        logger.error(e)


# @scheduler.scheduled_job('interval',id='import_func', seconds=5)
# def import_func():
#     if os.path.exists(os.sep.join([config.work_path,'test_script'])):
#         for root, dirs, files in os.walk(os.sep.join([config.work_path,'test_script'])):
#             for file in files:
#                 if os.path.splitext(file)[1] == '.py':
#                     importlib.import_module(f"test_script.{os.path.splitext(file)[0]}")
#     # logger.debug(fm.get_map())



def init_jobs():
    try:
        scheduler.start()
        logger.info(f"scheduler启动完成")
    except Exception as e:
        logger.error(f"job err =  {e}")


from setuptools import setup, find_packages
import shutil


try:
    shutil.rmtree("build")
    shutil.rmtree("dist")
except:
    pass


setup(name='script_client',
      version='1.1.1',
      author='TDC',
      author_email='tiandachun@meicai.cn',
      description="script client",
      long_description='''script client''',
      install_requires=['JPype1>=0.7.4',
                        "loguru>=0.3.2",
                        "requests>=2.22.0",
                        "tornado>=6.1",
                        "APScheduler>=3.7.0",
                        "psutil>=5.8.0",
                        ],
      license="MIT",
      packages=find_packages(),
      include_package_data=True,
      data_files=[
          ('script_client/templates', ['script_client/templates/upload.html']),
          ('script_client/static/css',
           ['script_client/static/css/bootstrap.min.css', 'script_client/static/css/fileinput.min.css']),
          ('script_client/static/js', ['script_client/static/js/bootstrap.min.js', 'script_client/static/js/fileinput.min.js',
                                    'script_client/static/js/jquery-3.4.1.min.js'])
      ]
      )



import os
import importlib
import configparser
from loguru import logger
from script_client import jobs
from script_client import func_map
from script_client import web_server
from script_client import run_script
from script_client.const import config as conf
from script_client.common.common_method import regist_app






print_str = '''
                               __              __                      __  __                        __     
                              /  |            /  |                    /  |/  |                      /  |    
  _______   _______   ______  $$/   ______   _$$ |_           _______ $$ |$$/   ______   _______   _$$ |_   
 /       | /       | /      \ /  | /      \ / $$   |         /       |$$ |/  | /      \ /       \ / $$   |  
/$$$$$$$/ /$$$$$$$/ /$$$$$$  |$$ |/$$$$$$  |$$$$$$/         /$$$$$$$/ $$ |$$ |/$$$$$$  |$$$$$$$  |$$$$$$/   
$$      \ $$ |      $$ |  $$/ $$ |$$ |  $$ |  $$ | __       $$ |      $$ |$$ |$$    $$ |$$ |  $$ |  $$ | __ 
 $$$$$$  |$$ \_____ $$ |      $$ |$$ |__$$ |  $$ |/  |      $$ \_____ $$ |$$ |$$$$$$$$/ $$ |  $$ |  $$ |/  |
/     $$/ $$       |$$ |      $$ |$$    $$/   $$  $$/       $$       |$$ |$$ |$$       |$$ |  $$ |  $$  $$/ 
$$$$$$$/   $$$$$$$/ $$/       $$/ $$$$$$$/     $$$$/         $$$$$$$/ $$/ $$/  $$$$$$$/ $$/   $$/    $$$$/  
                                  $$ |                                                                      
                                  $$ |                                                                      
                                  $$/                                                                          
'''



def _get_config(config_path):
    if config_path == '':
        config_path = 'config.ini'
    if not os.path.exists(config_path):
        logger.error(f"config.ini 不存在")
        return False
    try:
        config = configparser.ConfigParser()
        config.read(config_path, encoding="utf-8")
        conf.local_ip = config.get('conf', 'local_ip')
        conf.local_port = config.getint('conf', 'local_port')
        conf.client_name = config.get('conf', 'client_name')
        conf.regist_server_url = config.get('conf', 'regist_server_url')
        conf.heart_beat_url = config.get('conf', 'heart_beat_url')
        conf.result_url = config.get('conf', 'result_url')
        conf.http_request_timeout = config.getint('conf', 'http_request_timeout')
        conf.max_thread = config.getint('conf', 'max_thread')
        conf.max_task = config.getint('conf', 'max_task')
        conf.py_timeout = config.getint('conf', 'py_timeout')
        conf.robot_url = config.get('conf', 'robot_url')
        conf.work_path = os.getcwd()
        return True
    except Exception as e:
        logger.error(f"读取配置信息失败 {e}")
        return False


def _import_and_regist():
    if os.path.exists(os.sep.join([conf.work_path,'test_script'])):
            for root, dirs, files in os.walk(os.sep.join([conf.work_path,'test_script'])):
                for file in files:
                    if os.path.splitext(file)[1] == '.py':
                        importlib.import_module(f"test_script.{os.path.splitext(file)[0]}")
    try:
        d = func_map.fm.get_map()
        for k, v in d.items():
            for kk, vv in v.items():
                regist_app(k, kk)
        logger.info(f"app注册完成")
        return True
    except Exception as e:
        logger.error(f"app注册出错 = {e}")
        return False

def _run_threadpool():
    run_script.init_thread_pool()


def _run_job():
    jobs.init_jobs()


def _run_webserver():
    web_server.run_web_server()


def stop():
    web_server.stop_web_server()


def run(config_path=''):
    '''
    启动服务
    :param config_path: config路径，默认当前路径
    :return:
    '''
    f = os.sep.join([os.getcwd(), 'logs', 'script_client_{time}.log'])
    logger.add(f, rotation="1 days", retention="3 days")

    logger.info(print_str)
    if _get_config(config_path) and _import_and_regist():
        with open(r'pid.log', 'wb') as f:
            pid = os.getpid()
            f.write(str(pid).encode())
        _run_threadpool()
        _run_job()
        _run_webserver()









if __name__ == "__main__":
    run()




'''
样例
func_map = {
    'omc':{
        'path1':[函数对象,timeout],
        'path2':[函数对象,timeout]
    },
    'ofc':{
        'path1':[函数对象,timeout]
    }
}
'''


class FMap:
    def __init__(self):
        self.func_map = {}

    def add_map(self,app,path,func,timeout):
        if app not in self.func_map:
            self.func_map[app]= {path:[func,timeout]}
        else:
            t_dict = self.func_map[app]
            t_dict[path]=[func,timeout]
            self.func_map[app] = t_dict


    def get_map(self):
        return self.func_map





fm = FMap()